@@ -30,7 +30,7 @@
 #include <config.h>
 
 #include "rcsid.h"
-RCSID("$Id: passwd.c,v 1.7 1997/06/01 05:25:39 marekm Exp $")
+RCSID("$Id: passwd.c,v 1.1 2000/12/17 05:34:12 tom Exp $")
 
 #include "prototypes.h"
 #include "defines.h"
@@ -43,6 +43,14 @@ RCSID("$Id: passwd.c,v 1.7 1997/06/01 05:25:39 marekm Exp $")
 #include <userpw.h>
 #include <usersec.h>
 #include <userconf.h>
+# if !defined(SC_SYS_PASSWD) && defined(SEC_PASSWD)
+#  define SC_SYS_PASSWD SEC_PASSWD
+# endif
+#endif
+
+/* TJW: If PAM_MISC is not defined, we don't have do_pam_passwd */
+#if defined(PAM) && !defined(PAM_MISC)
+#undef PAM
 #endif
 
 #ifdef PAM
@@ -107,10 +115,11 @@ static int
 	eflg = 0,		/* -e - force password change */
 	kflg = 0,		/* -k - change only if expired */
 #endif
-#ifdef SHADOWPWD
+#if defined(SHADOWPWD) && defined(SP_EXTRA)
 	wflg = 0,		/* -w - set warning days */
 	iflg = 0,		/* -i - set inactive days */
 #endif
+	tflg = 0,		/* -t - update EPS tpasswd only */
 	qflg = 0,		/* -q - quiet mode */
 	aflg = 0,		/* -a - show status for all users */
 	Sflg = 0;		/* -S - show password status */
@@ -162,7 +171,7 @@ extern	int	pw_dbm_mode;
  */
 
 #define USAGE \
-	"usage: %s [ -f | -s ] [ name ]\n"
+	"usage: %s [ -f | -s ] [ -t ] [ name ]\n"
 #define ADMUSAGE \
 	"       %s [ -x max ] [ -n min ] [ -w warn ] [ -i inact ] name\n"
 #define ADMUSAGE2 \
@@ -300,6 +309,7 @@ reuse(pass, pw)
 #endif
 	return 0;
 }
+#endif /* !PAM */
 
 /*
  * new_password - validate old password and replace with new
@@ -335,7 +345,11 @@ new_password(pw)
 #endif
 		pass_max_len = getdef_num("PASS_MAX_LEN", 8);
 
-	if (! amroot && crypt_passwd[0]) {
+	if (! amroot
+#ifndef PAM
+	    && crypt_passwd[0]
+#endif
+	    ) {
 
 		/* EPS STUFF */
 
@@ -349,7 +363,8 @@ new_password(pw)
 		{
 		  if (retval == 0) retval = 1; else retval = 0;
 		}
-		else
+#ifndef PAM
+		else if(!tflg)
 		{
 		  if (strlen (clear) > pass_max_len)
 		  {
@@ -360,6 +375,7 @@ new_password(pw)
 		  cipher = pw_encrypt (clear, crypt_passwd);
 		  retval = strcmp (cipher, crypt_passwd);
 		}
+#endif /* PAM */
 
 		if (retval != 0)
 		{
@@ -402,7 +418,11 @@ new_password(pw)
 		bzero(cp, strlen(cp));
 
 		if (!amroot && getdef_bool("PASS_STRICT", 0) &&
-		    (!obscure(orig, pass, pw) || reuse(pass, pw))) {
+		    (!obscure(orig, pass, pw)
+#ifndef PAM
+		     || reuse(pass, pw)
+#endif
+		     )) {
 			printf (TRYAGAIN);
 			continue;
 		}
@@ -412,7 +432,11 @@ new_password(pw)
 		 */
 		if ((amroot || !getdef_bool("PASS_STRICT", 0)) && !warned &&
 		    getdef_bool("PASS_ALWAYS_WARN", 1)
-		    && (!obscure(orig, pass, pw) || reuse(pass, pw))) {
+		    && (!obscure(orig, pass, pw)
+#ifndef PAM
+			|| reuse(pass, pw)
+#endif
+			)) {
 			printf(PASSWARN);
 			warned++;
 			continue;
@@ -466,11 +490,14 @@ new_password(pw)
 #ifdef HAVE_LIBCRACK_HIST
 	HistUpdate(pw->pw_name, crypt_passwd);
 #endif
+#ifndef PAM
 	STRFCPY(crypt_passwd, cp);
+#endif
 
 	return 0;
 }
 
+#ifndef PAM
 /*
  * check_password - test a password to see if it can be changed
  *
@@ -665,8 +692,13 @@ print_status(pw)
 			date_to_str(sp->sp_lstchg * SCALE),
 			(sp->sp_min * SCALE) / DAY,
 			(sp->sp_max * SCALE) / DAY,
-			(sp->sp_warn * SCALE) / DAY,
-			(sp->sp_inact * SCALE) / DAY);
+#ifdef SP_EXTRA
+		        (sp->sp_warn * SCALE) / DAY,
+			(sp->sp_inact * SCALE) / DAY
+#else
+		        0, 0
+#endif /* SP_EXTRA */
+		       );
 	} else
 #endif
 	{
@@ -764,9 +796,17 @@ update_noshadow()
 	}
 	pw = pw_locate(name);
 	if (!pw) {
+#if 0
 		fprintf(stderr, "%s: user %s not found in /etc/passwd\n",
 			Prog, name);
 		fail_exit(1);
+#else
+		fprintf(stderr, "%s: warning: user %s not found in /etc/passwd\n",
+			Prog, name);
+		pw_close();
+		pw_unlock();
+		return;
+#endif
 	}
 	npw = __pw_dup(pw);
 	if (!npw)
@@ -895,10 +935,12 @@ update_shadow()
 		nsp->sp_max = (age_max * DAY) / SCALE;
 	if (nflg)
 		nsp->sp_min = (age_min * DAY) / SCALE;
+#ifdef SP_EXTRA
 	if (wflg)
 		nsp->sp_warn = (warn * DAY) / SCALE;
 	if (iflg)
 		nsp->sp_inact = (inact * DAY) / SCALE;
+#endif /* SP_EXTRA */
 	if (do_update_age)
 		nsp->sp_lstchg = time((time_t *) 0) / SCALE;
 	/*
@@ -994,6 +1036,7 @@ getnumber(str)
  *	-f	execute chfn command to interpret flags
  *	-s	execute chsh command to interpret flags
  *	-k	change password only if expired
+ *	-t	update EPS tpasswd only
  *
  *	(*) requires root permission to execute.
  *
@@ -1088,12 +1131,12 @@ main(argc, argv)
 	 */
 
 #ifdef SHADOWPWD
-#define FLAGS "adlquSekn:x:i:w:"
+#define FLAGS "adlqtuSekn:x:i:w:"
 #else
 #ifdef AGING
-#define FLAGS "adlquSekn:x:"
+#define FLAGS "adlqtuSekn:x:"
 #else
-#define FLAGS "adlquS"
+#define FLAGS "adlqtuS"
 #endif
 #endif
 	while ((flag = getopt(argc, argv, FLAGS)) != EOF) {
@@ -1110,7 +1153,7 @@ main(argc, argv)
 			nflg++;
 			anyflag = 1;
 			break;
-#ifdef	SHADOWPWD
+#if defined(SHADOWPWD) && defined(SP_EXTRA)
 		case 'w':
 			warn = getnumber(optarg);
 			if (warn >= -1)
@@ -1123,7 +1166,7 @@ main(argc, argv)
 				iflg++;
 			anyflag = 1;
 			break;
-#endif	/* SHADOWPWD */
+#endif	/* SHADOWPWD && SP_EXTRA */
 		case 'e':
 			eflg++;
 			anyflag = 1;
@@ -1136,6 +1179,9 @@ main(argc, argv)
 		case 'a':
 			aflg++;
 			break;
+		case 't':
+			tflg++;  /* ok for users */
+			break;
 		case 'q':
 			qflg++;  /* ok for users */
 			break;
@@ -1270,7 +1316,9 @@ main(argc, argv)
 		exit(0);
 	}
 
-#ifndef PAM
+#ifdef PAM
+	if(tflg) {
+#endif /* PAM */
 #ifdef SHADOWPWD
 	/*
 	 * The user name is valid, so let's get the shadow file
@@ -1291,6 +1339,8 @@ main(argc, argv)
 	 */
 
 	if (!anyflag) {
+#ifndef PAM
+		if(!tflg) {
 #ifdef AUTH_METHODS
 		if (strchr(cp, '@')) {
 			if (pw_auth(cp, name, PW_CHANGE, (char *)0)) {
@@ -1315,6 +1365,8 @@ main(argc, argv)
 #else
 		check_password(pw);
 #endif
+		}
+#endif /* !PAM */
 		/*
 		 * Let the user know whose password is being changed.
 		 */
@@ -1326,14 +1378,18 @@ main(argc, argv)
 			closelog();
 			exit(1);
 		}
+#ifndef PAM
 		do_update_pwd = 1;
+#endif
 		do_update_age = 1;
 	}
 
+#ifdef PAM
+	}
+#endif /* PAM */
 #ifdef AUTH_METHODS
 done:
 #endif
-#endif /* !PAM */
 	/*
 	 * Before going any further, raise the ulimit to prevent
 	 * colliding into a lowered ulimit, and set the real UID
@@ -1353,7 +1409,7 @@ done:
 	 * Don't set the real UID for PAM...
 	 */
 #ifdef PAM
-	if (!anyflag) {
+	if (!anyflag && !tflg) {
 		int flags = 0;
 
 		if (kflg)
@@ -1373,7 +1429,9 @@ done:
 		exit(1);
 	}
 #ifdef  HAVE_USERSEC_H
+	if(!tflg) {
 	update_userpw(pw->pw_passwd);
+	}
 #else  /* !HAVE_USERSEC_H */
 
 	if (!commonio_lock_all()) {
@@ -1382,12 +1440,14 @@ done:
 		exit(5);
 	}
 
+	if(!tflg) {
 #ifdef SHADOWPWD
 	if (access(SHADOW_FILE, 0) == 0)
 		update_shadow();
 	else
 #endif
 		update_noshadow();
+	}
 
 #endif /* !HAVE_USERSEC_H */
 
