@@ -35,6 +35,10 @@
 static char sccsid[] = "@(#)cmds.c	5.26 (Berkeley) 3/5/91";
 #endif /* not lint */
 
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
+
 /*
  * FTP User Program -- Command Routines.
  */
@@ -70,10 +74,12 @@ extern	char **ftpglob();
 extern	char *home;
 extern	char *remglob();
 extern	char *getenv();
+#ifdef NO_STRERROR
 #define strerror(error) (sys_errlist[error])
 #ifdef NEED_SYS_ERRLIST
 extern char *sys_errlist[];
-#endif
+#endif /* NEED_SYS_ERRLIST */
+#endif /* NO_STRERROR */
 
 extern off_t restart_point;
 extern char reply_string[];
@@ -193,6 +199,10 @@ setpeer(argc, argv)
 #endif
 #endif
 
+#ifndef NBBY	/* Default this to 8 */
+#define NBBY 8
+#endif /* NBBY */
+
 #if defined(unix) && (NBBY == 8 || defined(linux))
 /*
  * this ifdef is to keep someone form "porting" this to an incompatible
