@@ -41,6 +41,10 @@ char copyright[] =
 static char sccsid[] = "@(#)ftpd.c	5.40 (Berkeley) 7/2/91";
 #endif /* not lint */
 
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
+
 /*
  * FTP server.
  */
@@ -87,6 +91,10 @@ static char sccsid[] = "@(#)ftpd.c	5.40 (Berkeley) 7/2/91";
 #endif
 #include "pathnames.h"
 
+#ifndef MAXPATHLEN
+#define MAXPATHLEN 256
+#endif /* MAXPATHLEN */
+
 #ifndef L_SET
 #define L_SET 0
 #endif
@@ -94,10 +102,16 @@ static char sccsid[] = "@(#)ftpd.c	5.40 (Berkeley) 7/2/91";
 #define L_INCR 1
 #endif
 
+#ifndef NBBY	/* Default this to 8 */
+#define NBBY 8
+#endif /* NBBY */
+
+#ifdef NO_STRERROR
 #define strerror(error)	(sys_errlist[error])
 #ifdef NEED_SYS_ERRLIST
 extern char *sys_errlist[];
-#endif
+#endif /* NEED_SYS_ERRLIST */
+#endif /* NO_STRERROR */
 
 extern char *mktemp ();
 
@@ -132,7 +146,8 @@ KTEXT_ST ticket;
 MSG_DAT msg_data;
 Key_schedule schedule;
 int kerb_ok;	/* Kerberos authentication and authorization succeeded */
-char *keyfile = KEYFILE;
+char *keyfile;
+#define krb_get_err_text(R) krb_err_txt[R]
 #endif /* KERBEROS */
 
 #ifdef GSSAPI
@@ -176,7 +191,7 @@ int	debug;
 int	timeout = 900;    /* timeout after 15 minutes of inactivity */
 int	maxtimeout = 7200;/* don't allow idle time to be set beyond 2 hours */
 int	logging;
-int	authenticate;
+int	auth_required;
 int	guest;
 int	type;
 int	level;
@@ -239,6 +254,9 @@ main(argc, argv, envp)
 	char *cp;
 
 	debug = 0;
+#ifdef KERBEROS
+	keyfile = KEYFILE;
+#endif /* KERBEROS */
 
 #ifdef SETPROCTITLE
 	/*
@@ -266,7 +284,7 @@ main(argc, argv, envp)
 			break;
 
 		case 'a':
-			authenticate = 1;
+			auth_required = 1;
 			break;
 
 		case 'p':
@@ -627,7 +645,7 @@ user(name)
 	if (auth_type && strcmp(auth_type, "KERBEROS_V4") == 0) {
 		char buf[FTP_BUFSIZ];
 		kerb_ok = kuserok(&kdata,name) == 0;
-		if (! kerb_ok && authenticate) {
+		if (! kerb_ok && auth_required) {
 			reply(530, "User %s access denied.", name);
 			if (logging)
 				syslog(LOG_NOTICE,
@@ -649,7 +667,7 @@ user(name)
 	if (auth_type && strcmp(auth_type, "GSSAPI") == 0) {
 		char buf[FTP_BUFSIZ];
 		gss_ok = ftpd_userok(&client_name, name) == 0;
-		if (! gss_ok && authenticate) {
+		if (! gss_ok && auth_required) {
 			reply(530, "User %s access denied.", name);
 			if (logging)
 				syslog(LOG_NOTICE,
@@ -668,7 +686,7 @@ user(name)
 	} else
 #endif /* GSSAPI */
 	/* Other auth types go here ... */
-	if (authenticate) {
+	if (auth_required) {
 		reply(530, "User %s access denied: authentication required.",
 		      name);
 		if (logging)
@@ -1539,15 +1557,16 @@ reply(n, fmt, p0, p1, p2, p3, p4, p5)
 		else in[0] = '\0';
 		strcat(in, buf);
 #ifdef SRP
-	if (strcmp(auth_type, "SRP") == 0)
-            if ((length = srp_encode (level == PROT_P, (unsigned char *) in,
-            (unsigned char *) out, strlen (in))) < 0)
-            {
-              syslog (LOG_ERR, "SRP failed to encode message\n");
-            }
+		if (strcmp(auth_type, "SRP") == 0) {
+		  if ((length = srp_encode (level == PROT_P, (unsigned char *) in,
+					    (unsigned char *) out, strlen (in))) < 0)
+		    {
+		      syslog (LOG_ERR, "SRP failed to encode message\n");
+		    }
+		}
 #endif
 #ifdef KERBEROS
-		if (strcmp(auth_type, "KERBEROS_V4") == 0)
+		if (strcmp(auth_type, "KERBEROS_V4") == 0) {
 		  if ((length = level == PROT_P ?
 		    krb_mk_priv((unsigned char *)in, (unsigned char *)out,
 				strlen(in), schedule, &kdata.session,
@@ -1558,7 +1577,8 @@ reply(n, fmt, p0, p1, p2, p3, p4, p5)
 			syslog(LOG_ERR, "krb_mk_%s failed for KERBEROS_V4",
 					level == PROT_P ? "priv" : "safe");
 			fputs(in,stdout);
-		  } else
+		  }
+		}
 #endif /* KERBEROS */
 #ifdef GSSAPI
 		/* reply (based on level) */
