@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1997-1999  The Stanford SRP Authentication Project
+ * Copyright (c) 1997-2001  The Stanford SRP Authentication Project
  * All Rights Reserved.
  *
  * Permission is hereby granted, free of charge, to any person obtaining
@@ -43,7 +43,7 @@
 #include <config.h>
 #endif
 
-#ifdef SRP
+#ifdef HAVE_SRP
 #include <sys/types.h>
 #include <arpa/telnet.h>
 #include <stdio.h>
@@ -63,8 +63,7 @@
 #include "misc.h"
 
 #include "t_pwd.h"
-#include "t_server.h"
-#include "t_client.h"
+#include "srp.h"
 
 extern auth_debug_mode;
 
@@ -82,15 +81,13 @@ static unsigned char str_data[1024] = { IAC, SB, TELOPT_AUTHENTICATION, 0,
 static unsigned char str_name[1024] = { IAC, SB, TELOPT_AUTHENTICATION,
 					TELQUAL_NAME, };
 
-static struct t_server * ts = NULL;
-static struct t_client * tc = NULL;
-/*
-static struct t_pw * tpw = NULL;
-static struct t_conf * tconf = NULL;
-*/
+static SRP * s_srp = NULL;
+static cstr * s_key = NULL;
+static SRP * c_srp = NULL;
+static cstr * c_key = NULL;
 
 static int waitresponse = 0;	/* Flag to indicate readiness for response */
-static struct t_num * B;	/* Holder for B */
+static cstr * B = NULL;		/* Holder for B */
 
 #define PWD_SZ 128
 
@@ -219,9 +216,9 @@ srp_is(ap, data, cnt)
 {
   char pbuf[2 * MAXPARAMLEN + 5];
   char * ptr;
-  struct t_num A;
   char hexbuf[MAXHEXPARAMLEN];
-  struct passwd * pass;
+  struct t_passwd * pass;
+  cstr * resp;
 /*
   FILE * passfp = NULL;
   FILE * conffp = NULL;
@@ -239,9 +236,9 @@ srp_is(ap, data, cnt)
   switch(*data++) {
   case SRP_AUTH:
     /* Send parameters back to client */
-    if(ts != NULL) {
-      t_serverclose(ts);
-      ts = NULL;
+    if(s_srp != NULL) {
+      SRP_free(s_srp);
+      s_srp = NULL;
     }
     if(!UserNameRequested) {
       if (auth_debug_mode)
@@ -268,7 +265,17 @@ srp_is(ap, data, cnt)
     }
     ts = t_serveropen(UserNameRequested, tpw, tconf);
 */
+    /*
     ts = t_serveropen(UserNameRequested);
+    */
+    s_srp = SRP_new(SRP_RFC2945_server_method());
+    if(s_srp == NULL) {
+      if (auth_debug_mode)
+	printf("Error initializing SRP server\r\n");
+      Data(ap, SRP_REJECT, (void *) "SRP server init failed", -1);
+      break;
+    }
+    pass = gettpnam(UserNameRequested);
 /*
     t_closepw(tpw);
     if(passfp)
@@ -280,38 +287,55 @@ srp_is(ap, data, cnt)
     tconf = NULL;
 */
 
-    if(ts == NULL) {
+    if(pass == NULL) {
       if (auth_debug_mode)
 	printf("User %s not found\r\n", UserNameRequested);
       Data(ap, SRP_REJECT, (void *) "Password not set", -1);
       break;
     }
+    if(SRP_set_username(s_srp, UserNameRequested) != SRP_SUCCESS ||
+       SRP_set_params(s_srp, pass->tc.modulus.data, pass->tc.modulus.len,
+		      pass->tc.generator.data, pass->tc.generator.len,
+		      pass->tp.salt.data, pass->tp.salt.len) != SRP_SUCCESS ||
+       SRP_set_authenticator(s_srp, pass->tp.password.data,
+			     pass->tp.password.len) != SRP_SUCCESS) {
+      if (auth_debug_mode)
+	printf("Error initializing SRP parameters\r\n");
+      Data(ap, SRP_REJECT, (void *) "SRP parameter init failed", -1);
+      break;
+    }
+
     ptr = pbuf;
 
-    encode_length(ptr, ts->n.len);
+    encode_length(ptr, pass->tc.modulus.len);
     ptr += 2;
-    memcpy(ptr, ts->n.data, ts->n.len);
-    ptr += ts->n.len;
+    memcpy(ptr, pass->tc.modulus.data, pass->tc.modulus.len);
+    ptr += pass->tc.modulus.len;
 
-    encode_length(ptr, ts->g.len);
+    encode_length(ptr, pass->tc.generator.len);
     ptr += 2;
-    memcpy(ptr, ts->g.data, ts->g.len);
-    ptr += ts->g.len;
+    memcpy(ptr, pass->tc.generator.data, pass->tc.generator.len);
+    ptr += pass->tc.generator.len;
 
-    encode_length(ptr, ts->s.len);
+    encode_length(ptr, pass->tp.salt.len);
     ptr += 2;
-    memcpy(ptr, ts->s.data, ts->s.len);
-    ptr += ts->s.len;
+    memcpy(ptr, pass->tp.salt.data, pass->tp.salt.len);
+    ptr += pass->tp.salt.len;
 
-    Data(ap, SRP_PARAMS, pbuf, ptr - pbuf);
+    Data(ap, SRP_PARAMS, pbuf, ptr - pbuf);  /* Ideally, this is flushed */
 
-    B = t_servergenexp(ts);
+    if(SRP_gen_pub(s_srp, &B) != SRP_SUCCESS) {
+      if (auth_debug_mode)
+	printf("Error generating SRP public value\r\n");
+      Data(ap, SRP_REJECT, (void *) "SRP_gen_pub failed", -1);
+      break;
+    }
 
     break;
 
   case SRP_EXP:
     /* Client is sending A to us.  Compute challenge and expected response. */
-    if(ts == NULL || B == NULL) {
+    if(s_srp == NULL || B == NULL) {
       if (auth_debug_mode)
 	printf("Protocol error: SRP_EXP unexpected\r\n");
       Data(ap, SRP_REJECT, (void *) "Protocol error: unexpected EXP", -1);
@@ -319,13 +343,14 @@ srp_is(ap, data, cnt)
     }
 
     /* Wait until now to send B, since it contains the key to "u" */
-    Data(ap, SRP_CHALLENGE, B->data, B->len);
+    Data(ap, SRP_CHALLENGE, B->data, B->length);
+    cstr_free(B);
     B = NULL;
 
     if ( ap->way & AUTH_ENCRYPT_MASK ) {
       type_check[0] = (unsigned char) ap->type;
       type_check[1] = (unsigned char) ap->way;
-      t_serveraddexdata(ts,type_check,2);
+      SRP_add_ex_data(s_srp, type_check, 2);
     }
  
 #ifdef TLS
@@ -336,17 +361,13 @@ srp_is(ap, data, cnt)
      */
     if(tls_active && (ap->way & AUTH_ENCRYPT_MASK) == AUTH_ENCRYPT_START_TLS) {
       flen = tls_get_client_finished(fin_buf, sizeof(fin_buf));
-      t_serveraddexdata(ts, fin_buf, flen);
+      SRP_add_ex_data(s_srp, fin_buf, flen);
       flen = tls_get_server_finished(fin_buf, sizeof(fin_buf));
-      t_serveraddexdata(ts, fin_buf, flen);
+      SRP_add_ex_data(s_srp, fin_buf, flen);
     }
 #endif /* TLS */
 
-    A.data = data;
-    A.len = cnt;
-    ptr = t_servergetkey(ts, &A);
-
-    if(ptr == NULL) {
+    if(SRP_compute_key(s_srp, &s_key, data, cnt) != SRP_SUCCESS) {
       if (auth_debug_mode)
 	printf("Security alert: Trivial session key attempted\r\n");
       Data(ap, SRP_REJECT, (void *) "Trivial session key detected", -1);
@@ -358,7 +379,7 @@ srp_is(ap, data, cnt)
 
   case SRP_RESPONSE:
     /* Got the response; see if it's correct */
-    if(ts == NULL || !waitresponse) {
+    if(s_srp == NULL || !waitresponse) {
       if (auth_debug_mode)
 	printf("Protocol error: SRP_RESPONSE unexpected\r\n");
       Data(ap, SRP_REJECT, (void *) "Protocol error: unexpected RESPONSE", -1);
@@ -372,14 +393,24 @@ srp_is(ap, data, cnt)
       break;
     }
 
-    if(t_serververify(ts, data) == 0) {
-      Data(ap, SRP_ACCEPT, t_serverresponse(ts), RESPONSE_LEN);
+    if(SRP_verify(s_srp, data, cnt) == SRP_SUCCESS) {
+      resp = cstr_new();
+      if(SRP_respond(s_srp, &resp) != SRP_SUCCESS) {
+	if (auth_debug_mode)
+	  printf("Error computing response\r\n");
+	Data(ap, SRP_REJECT, (void *) "Error computing response", -1);
+	break;
+      }
+      Data(ap, SRP_ACCEPT, resp->data, resp->length);
+      cstr_free(resp);
 
 #ifdef	ENCRYPTION
       skey.type = SK_GENERIC;
-      skey.length = 40;
-      skey.data = ts->session_key;
+      skey.length = s_key->length;
+      skey.data = s_key->data;
       encrypt_session_key(&skey, 1);
+      cstr_clear_free(s_key);
+      s_key = NULL;
 #endif
 
       auth_finished(ap, AUTH_VALID);
@@ -409,8 +440,8 @@ srp_reply(ap, data, cnt)
   struct t_num g;
   struct t_num s;
 
-  struct t_num B;
-  struct t_num * A;
+  cstr * resp;
+  cstr * A;
 
   char hexbuf[MAXHEXPARAMLEN];
   int pflag;
@@ -434,13 +465,13 @@ srp_reply(ap, data, cnt)
     auth_send_retry();
     break;
   case SRP_ACCEPT:
-    if(tc == NULL || cnt < RESPONSE_LEN || !waitresponse) {
+    if(c_srp == NULL || cnt < RESPONSE_LEN || !waitresponse) {
       if (auth_debug_mode)
 	printf("Protocol error\r\n");
       break;
     }
 
-    if(t_clientverify(tc, data) == 0) {
+    if(SRP_verify(c_srp, data, cnt) == SRP_SUCCESS) {
       printf("[ SRP authentication successful ]\r\n");
 #ifdef TLS
       if(tls_active &&
@@ -452,9 +483,11 @@ srp_reply(ap, data, cnt)
 
 #ifdef	ENCRYPTION
       skey.type = SK_GENERIC;
-      skey.length = SESSION_KEY_LEN;
-      skey.data = tc->session_key;
+      skey.length = c_key->length;
+      skey.data = c_key->data;
       encrypt_session_key(&skey, 0);
+      cstr_clear_free(c_key);
+      c_key = NULL;
 #endif
 
       auth_finished(ap, AUTH_VALID);
@@ -464,8 +497,8 @@ srp_reply(ap, data, cnt)
       auth_send_retry();
       break;
     }
-    t_clientclose(tc);
-    tc = NULL;
+    SRP_free(c_srp);
+    c_srp = NULL;
     break;
   case SRP_PARAMS:
     if(!UserNameRequested) {
@@ -512,25 +545,37 @@ srp_reply(ap, data, cnt)
     data += s.len;
     cnt -= s.len;
 
-    tc = t_clientopen(UserNameRequested, &n, &g, &s);
-    if(tc == NULL) {
+    c_srp = SRP_new(SRP_RFC2945_client_method());
+    if(c_srp == NULL ||
+       SRP_set_username(c_srp, UserNameRequested) != SRP_SUCCESS ||
+       SRP_set_params(c_srp, n.data, n.len, g.data, g.len, s.data, s.len) !=
+       SRP_SUCCESS) {
       printf("[ Parameter initialization error ]\r\n");
       auth_send_retry();
       break;
     }
 
-    A = t_clientgenexp(tc);
+    A = cstr_new();
+    if(SRP_gen_pub(c_srp, &A) != SRP_SUCCESS) {
+      printf("[ Error generating key exchange ]\r\n");
+      auth_send_retry();
+      break;
+    }
 
-    Data(ap, SRP_EXP, A->data, A->len);
+    Data(ap, SRP_EXP, A->data, A->length);
+    cstr_free(A);
 
     local_des_read_pw_string(user_passwd, sizeof(user_passwd) - 1, "SRP Password: ", 0);
-    t_clientpasswd(tc, user_passwd);
+    if(SRP_set_auth_password(c_srp, user_passwd) != SRP_SUCCESS) {
+      printf("[ Error setting client password ]\r\n");
+      auth_send_retry();
+    }
     memset(user_passwd, 0, sizeof(user_passwd));
 
     break;
 
   case SRP_CHALLENGE:
-    if(tc == NULL) {
+    if(c_srp == NULL) {
       if (auth_debug_mode)
 	printf("Protocol error\r\n");
       break;
@@ -539,7 +584,7 @@ srp_reply(ap, data, cnt)
     if ( ap->way & AUTH_ENCRYPT_MASK ) {
       type_check[0] = (unsigned char) ap->type;
       type_check[1] = (unsigned char) ap->way;
-      t_clientaddexdata(tc,type_check,2);
+      SRP_add_ex_data(c_srp, type_check, 2);
     }
 
 #ifdef TLS
@@ -550,22 +595,31 @@ srp_reply(ap, data, cnt)
 	auth_send_retry();
 	break;
       }
-      t_clientaddexdata(tc, fin_buf, flen);
+      SRP_add_ex_data(c_srp, fin_buf, flen);
       flen = tls_get_server_finished(fin_buf, sizeof(fin_buf));
       if(flen <= 0) {
 	printf("ERROR: no TLS server Finished message\r\n");
 	auth_send_retry();
 	break;
       }
-      t_clientaddexdata(tc, fin_buf, flen);
+      SRP_add_ex_data(c_srp, fin_buf, flen);
     }
 #endif /* TLS */
 
-    B.data = data;
-    B.len = cnt;
-    t_clientgetkey(tc, &B);
+    if(SRP_compute_key(c_srp, &c_key, data, cnt) != SRP_SUCCESS) {
+      printf("ERROR: unable to compute client key\r\n");
+      auth_send_retry();
+      break;
+    }
 
-    Data(ap, SRP_RESPONSE, t_clientresponse(tc), RESPONSE_LEN);
+    resp = cstr_new();
+    if(SRP_respond(c_srp, &resp) != SRP_SUCCESS) {
+      printf("ERROR: unable to compute client response\r\n");
+      auth_send_retry();
+      break;
+    }
+    Data(ap, SRP_RESPONSE, resp->data, resp->length);
+    cstr_free(resp);
     waitresponse = 1;
 
     break;
