@@ -1989,6 +1989,10 @@ start_login(host, autologin, name)
 		close(pty);
 #endif
 	closelog();
+#ifdef KRB5
+	if (autologin != AUTH_VALID)
+		kerberos5_cleanup();
+#endif /* KRB5 */
 	execv(login_program, argv);
 
 	syslog(LOG_ERR, "%s: %m", login_program);
