@@ -62,6 +62,7 @@ static char copyright[] =
 #include <dirent.h>
 #include <fcntl.h>
 #include <netinet/in.h>
+#include <sys/stat.h>
 #include <openssl/pem.h>
 #endif /* TLS_SESSION_FILE_CACHE */
 #include "tls_dh.h"
@@ -75,8 +76,10 @@ static char copyright[] =
 #endif
 
 #define DEFRSACERTFILE		"telnetd-rsa.pem"
+#define DEFRSACERTCHAINFILE	"telnetd-rsa-chain.pem"
 #define DEFRSAKEYFILE		"telnetd-rsa-key.pem" 
 #define DEFDSACERTFILE		"telnetd-dsa.pem"
+#define DEFDSACERTCHAINFILE	"telnetd-dsa-chain.pem"
 #define DEFDSAKEYFILE		"telnetd-dsa-key.pem" 
 #define DEFCRLFILE		"telnetd-crl.pem"
 #define DEFDHPARAMFILE		"telnetd-dhparam.pem"
@@ -97,13 +100,17 @@ SSL_CTX	*ssl_ctx = NULL;
 X509_STORE *crl_store = NULL;
 char 	*tls_rsa_key_file = NULL;
 char	*tls_rsa_cert_file = NULL;
+char	*tls_rsa_cert_chain_file = NULL;
 char 	*tls_dsa_key_file = NULL;
 char	*tls_dsa_cert_file = NULL;
+char	*tls_dsa_cert_chain_file = NULL;
 char	*tls_crl_file = NULL;
 char	*tls_crl_dir = NULL;
 char	*tls_dhparam_file = NULL;
 char	*tls_rand_file = NULL;
 char	*tls_cipher_list = NULL;
+DH	*tmp_dh = NULL;
+RSA	*tmp_rsa = NULL;
 
 /* we need this so we don't mix static and malloc'ed strings */
 void tls_set_defaults(void)
@@ -112,10 +119,14 @@ void tls_set_defaults(void)
     	strcpy(tls_rsa_key_file, DEFRSAKEYFILE);
     if (tls_rsa_cert_file = malloc(strlen(DEFRSACERTFILE) + 1))
     	strcpy(tls_rsa_cert_file, DEFRSACERTFILE);
+    if (tls_rsa_cert_chain_file = malloc(strlen(DEFRSACERTCHAINFILE) + 1))
+    	strcpy(tls_rsa_cert_chain_file, DEFRSACERTCHAINFILE);
     if (tls_dsa_key_file = malloc(strlen(DEFDSAKEYFILE) + 1))
     	strcpy(tls_dsa_key_file, DEFDSAKEYFILE);
     if (tls_dsa_cert_file = malloc(strlen(DEFDSACERTFILE) + 1))
     	strcpy(tls_dsa_cert_file, DEFDSACERTFILE);
+    if (tls_dsa_cert_chain_file = malloc(strlen(DEFDSACERTCHAINFILE) + 1))
+    	strcpy(tls_dsa_cert_chain_file, DEFDSACERTCHAINFILE);
     if (tls_crl_file = malloc(strlen(DEFCRLFILE) + 1))
     	strcpy(tls_crl_file, DEFCRLFILE);
     if (tls_crl_dir = malloc(strlen(X509_get_default_cert_area()) + 5))
@@ -128,38 +139,35 @@ void tls_set_defaults(void)
 
 DH *tmp_dh_cb(SSL *ssl, int is_export, int keylength)
 {
-    static DH *dh = NULL;
     FILE *fp;
 
-    if (!dh) {
+    if (!tmp_dh) {
     	/* first try any 'tls_dhparam_file', else use built-in dh params */
     	if (tls_dhparam_file && (fp = fopen(tls_dhparam_file, "r"))) {
-	    dh = PEM_read_DHparams(fp, NULL, NULL, NULL);
+	    tmp_dh = PEM_read_DHparams(fp, NULL, NULL, NULL);
 	    fclose(fp);
-	    if (dh)
-	    	return dh;
+	    if (tmp_dh)
+	    	return tmp_dh;
 	}
 	switch (keylength) {
-	    case 512:	return dh = get_dh512();
-	    case 768:	return dh = get_dh768();
-	    case 1024:	return dh = get_dh1024();
-	    case 1536:	return dh = get_dh1536();
-	    case 2048:	return dh = get_dh2048();
-	    default:	return dh = get_dh1024();
+	    case 512:	return tmp_dh = get_dh512();
+	    case 768:	return tmp_dh = get_dh768();
+	    case 1024:	return tmp_dh = get_dh1024();
+	    case 1536:	return tmp_dh = get_dh1536();
+	    case 2048:	return tmp_dh = get_dh2048();
+	    default:	return tmp_dh = get_dh1024();
 	}
     }
     else
-    	return dh;
+    	return tmp_dh;
 }
 
 #ifndef NO_RSA
 RSA *tmp_rsa_cb(SSL *s, int is_export, int keylength)
 {
-    static RSA *rsa_tmp = NULL;
-
-    if (!rsa_tmp)
-	rsa_tmp = RSA_generate_key(keylength, RSA_F4, NULL, NULL);
-    return rsa_tmp;
+    if (!tmp_rsa)
+	tmp_rsa = RSA_generate_key(keylength, RSA_F4, NULL, NULL);
+    return tmp_rsa;
 }
 #endif /* NO_RSA */
 
@@ -175,6 +183,12 @@ void tls_optarg(char *optarg)
 	    if (tls_rsa_cert_file = malloc(strlen(p) + 1))
 	    	strcpy(tls_rsa_cert_file, p);
 	}
+	else if (!strcmp(optarg, "chain") || !strcmp(optarg, "rsachain")) {
+	    if (tls_rsa_cert_chain_file)
+	    	free(tls_rsa_cert_chain_file);
+	    if (tls_rsa_cert_chain_file = malloc(strlen(p) + 1))
+	    	strcpy(tls_rsa_cert_chain_file, p);
+	}
 	else if (!strcmp(optarg, "key") || !strcmp(optarg, "rsakey")) {
 	    if (tls_rsa_key_file)
 	    	free(tls_rsa_key_file);
@@ -187,6 +201,12 @@ void tls_optarg(char *optarg)
 	    if (tls_dsa_cert_file = malloc(strlen(p) + 1))
 	    	strcpy(tls_dsa_cert_file, p);
 	}
+	else if (!strcmp(optarg, "dsachain")) {
+	    if (tls_dsa_cert_chain_file)
+	    	free(tls_dsa_cert_chain_file);
+	    if (tls_dsa_cert_chain_file = malloc(strlen(p) + 1))
+	    	strcpy(tls_dsa_cert_chain_file, p);
+	}
 	else if (!strcmp(optarg, "dsakey")) {
 	    if (tls_dsa_key_file)
 	    	free(tls_dsa_key_file);
@@ -631,6 +651,8 @@ int tls_sfc_server_load(SSL_CTX *sc)
 			unlink(filename);
 		    else
 			SSL_CTX_add_session(sc, sess);
+		    /* dec the ref counter in sess so it will eventually be freed */
+		    SSL_SESSION_free(sess);
 		}
 		fclose(file);
 	    }
@@ -690,12 +712,15 @@ int tls_init(void)
 
     /* let's find out which files are available */
     check_file(&tls_rsa_cert_file);
+    check_file(&tls_rsa_cert_chain_file);
     check_file(&tls_rsa_key_file);
     check_file(&tls_dsa_cert_file);
+    check_file(&tls_dsa_cert_chain_file);
     check_file(&tls_dsa_key_file);
     check_file(&tls_crl_file);
     check_file(&tls_dhparam_file);
-    if (!tls_rsa_cert_file && !tls_dsa_cert_file) {
+    if (!tls_rsa_cert_file && !tls_rsa_cert_chain_file &&
+		!tls_dsa_cert_file && !tls_dsa_cert_chain_file) {
 #ifdef AUTHENTICATION
 	tls_anon = 1;
 #else
@@ -718,14 +743,29 @@ int tls_init(void)
 		(char *)ERR_error_string(ERR_get_error(), NULL));
 	    return 3;
 	}
-	SSL_CTX_set_tmp_rsa_callback(ssl_ctx, tmp_rsa_cb);
+	if (!tls_rsa_key_file)
+	    tls_rsa_key_file = tls_rsa_cert_file;
+    }
+    /* if you are using a chain file, the server's cert is supposed to be included
+     * first in the file, and takes presence over a cert file.
+     */
+    if (tls_rsa_cert_chain_file) {
+	err = SSL_CTX_use_certificate_chain_file(ssl_ctx, tls_rsa_cert_chain_file);
+	if (err <= 0) {
+	    fprintf(stderr, "SSL_CTX_use_certificate_chain_file(%s) %s\n",
+		    tls_rsa_cert_chain_file,
+		    (char *)ERR_error_string(ERR_get_error(), NULL));
+	    return 4;
+	}
+	if (!tls_rsa_key_file || tls_rsa_key_file == tls_rsa_cert_file)
+	    tls_rsa_key_file = tls_rsa_cert_chain_file;
     }
     if (tls_rsa_key_file) {
 	err = SSL_CTX_use_PrivateKey_file(ssl_ctx, tls_rsa_key_file, X509_FILETYPE_PEM);
 	if (err <= 0) {
 	    fprintf(stderr, "SSL_CTX_use_PrivateKey_file(%s) %s\n", tls_rsa_key_file,
 	    	(char *)ERR_error_string(ERR_get_error(), NULL));
-	    return 4;
+	    return 5;
 	}
     }
 #endif /* NO_RSA */
@@ -734,17 +774,31 @@ int tls_init(void)
 	if (err <= 0) {
 	    fprintf(stderr, "SSL_CTX_use_certificate_file(%s) %s\n", tls_dsa_cert_file,
 		(char *)ERR_error_string(ERR_get_error(), NULL));
-	    return 5;
+	    return 6;
 	}
+	if (!tls_dsa_key_file)
+	    tls_dsa_key_file = tls_dsa_cert_file;
+    }
+    if (tls_dsa_cert_chain_file) {
+	err = SSL_CTX_use_certificate_chain_file(ssl_ctx, tls_dsa_cert_chain_file);
+	if (err <= 0) {
+	    fprintf(stderr, "SSL_CTX_use_certificate_chain_file(%s) %s\n",
+		    tls_dsa_cert_chain_file,
+		    (char *)ERR_error_string(ERR_get_error(), NULL));
+	    return 7;
+	}
+	if (!tls_dsa_key_file || tls_dsa_key_file == tls_dsa_cert_file)
+	    tls_dsa_key_file = tls_dsa_cert_chain_file;
     }
     if (tls_dsa_key_file) {
 	err = SSL_CTX_use_PrivateKey_file(ssl_ctx, tls_dsa_key_file, X509_FILETYPE_PEM);
 	if (err <= 0) {
 	    fprintf(stderr, "SSL_CTX_use_PrivateKey_file(%s) %s\n", tls_dsa_key_file,
 	    	(char *)ERR_error_string(ERR_get_error(), NULL));
-	    return 6;
+	    return 8;
 	}
     }
+    SSL_CTX_set_tmp_rsa_callback(ssl_ctx, tmp_rsa_cb);
     SSL_CTX_set_tmp_dh_callback(ssl_ctx, tmp_dh_cb);
 
     /* set up the CRL */
@@ -765,7 +819,7 @@ int tls_init(void)
     if (!ssl) {
 	fprintf(stderr, "SSL_new() %s\n",
 		(char *)ERR_error_string(ERR_get_error(), NULL));
-	return 4;
+	return 9;
     }
     SSL_set_fd(ssl, net);
 
@@ -804,11 +858,18 @@ tls_userid_from_client_cert(void)
     }
 #ifdef TLS_KRB5
     else if (tls_is_krb5()) {
-        krb5_context kcontext;
-        krb5_principal user;
+        krb5_context kcontext = NULL;
+        krb5_principal user = NULL;
         krb5_error_code code;
 
         r = NULL;
+
+        if (ssl->kssl_ctx == NULL)
+            goto k5_cleanup;
+
+        if (ssl->kssl_ctx->client_princ == NULL)
+            goto k5_cleanup;
+
         code = krb5_init_context(&kcontext);
         if (code) goto k5_cleanup;
 
@@ -824,7 +885,8 @@ tls_userid_from_client_cert(void)
     }
 #endif /* TLS_KRB5 */
     else
-	return r = NULL;
+	r = NULL;
+    return r;
 }
 
 int tls_is_user_valid(char *user)
@@ -845,10 +907,16 @@ int tls_is_user_valid(char *user)
 #ifdef TLS_KRB5
     if ( tls_is_krb5() )
     {
-        krb5_context kcontext;
-        krb5_principal princ;
+        krb5_context kcontext = NULL;
+        krb5_principal princ = NULL;
         krb5_error_code code;
 
+        if (ssl->kssl_ctx == NULL)
+            goto k5_cleanup;
+
+        if (ssl->kssl_ctx->client_princ == NULL)
+            goto k5_cleanup;
+
         code = krb5_init_context(&kcontext);
         if (code) goto k5_cleanup;
 
@@ -921,9 +989,6 @@ void tls_cleanup(void)
 	crl_store = NULL;
     }
     if (ssl) {
-#ifdef TLS_SESSION_FILE_CACHE
-/*XXX	tls_sfc_server_save(ssl); */
-#endif /* TLS_SESSION_FILE_CACHE */
 	SSL_free(ssl);
 	ssl = NULL;
     }
@@ -931,6 +996,17 @@ void tls_cleanup(void)
 	SSL_CTX_free(ssl_ctx);
 	ssl_ctx = NULL;
     }
+    if (tmp_dh) {
+	DH_free(tmp_dh);
+	tmp_dh = NULL;
+    }
+    if (tmp_rsa) {
+	RSA_free(tmp_rsa);
+	tmp_rsa = NULL;
+    }
+    ERR_free_strings();
+    ERR_remove_state(0);
+    EVP_cleanup();	/* release the stuff allocated by SSL_library_init() */
     if (tls_rsa_key_file) {
     	if (tls_rsa_key_file != tls_rsa_cert_file)
 	    free(tls_rsa_key_file);
@@ -940,6 +1016,10 @@ void tls_cleanup(void)
     	free(tls_rsa_cert_file);
 	tls_rsa_cert_file = NULL;
     }
+    if (tls_rsa_cert_chain_file) {
+    	free(tls_rsa_cert_chain_file);
+	tls_rsa_cert_chain_file = NULL;
+    }
     if (tls_dsa_key_file) {
     	if (tls_dsa_key_file != tls_dsa_cert_file)
 	    free(tls_dsa_key_file);
@@ -949,6 +1029,10 @@ void tls_cleanup(void)
     	free(tls_dsa_cert_file);
 	tls_dsa_cert_file = NULL;
     }
+    if (tls_dsa_cert_chain_file) {
+    	free(tls_dsa_cert_chain_file);
+	tls_dsa_cert_chain_file = NULL;
+    }
     if (tls_dhparam_file) {
     	free(tls_dhparam_file);
 	tls_dhparam_file = NULL;
